﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.Robocode {
  [StorableClass]
  public class IfThenElseStat : CodeNode {
    public override int MinimumArity { get { return 2; } }
    public override int MaximumArity { get { return 3; } }

    [Storable]
    public override string Prefix { get; set; }

    [Storable]
    public override string Suffix { get; set; }

    [StorableConstructor]
    protected IfThenElseStat(bool deserializing) : base(deserializing) { }
    protected IfThenElseStat(IfThenElseStat original, Cloner cloner) : base(original, cloner) { }

    public IfThenElseStat() : base("IfThenElseStat", "An if statement.") { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new IfThenElseStat(this, cloner);
    }

    public override string Interpret(ISymbolicExpressionTreeNode node, IEnumerable<ISymbolicExpressionTreeNode> children) {
      ISymbolicExpressionTreeNode condition = null, truePart = null, falsePart = null;
      var enumerator = children.GetEnumerator();
      int childCount = 0;
      while (enumerator.MoveNext()) {
        childCount++;
        switch (childCount) {
          case 1: condition = enumerator.Current; break;
          case 2: truePart = enumerator.Current; break;
          case 3: falsePart = enumerator.Current; break;
          default: throw new ArgumentException("Unexpected number of children. Expected a maximum of 3 children.");
        }
      }
      if (childCount < 2) throw new ArgumentException("Unexpected number of children. Expected at least 2 children.");

      var parts = new[] {
        ((CodeNode)condition.Symbol).Interpret(condition, condition.Subtrees),
        ((CodeNode)truePart.Symbol).Interpret(truePart, truePart.Subtrees),
        ((CodeNode)falsePart.Symbol).Interpret(falsePart, falsePart.Subtrees)
      };

      string result = "if (" + parts[0] + ") {" + parts[1] + "}";
      if (!string.IsNullOrEmpty(parts[2]))
        result += " else {" + result[2] + "}";
      return result;
    }
  }
}
